#
# Copyright 2014 the original author or authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
import pkg_resources

import mock

from cabalgata.silla import plugins
from cabalgata.silla.util import disk


def plugin_entry_points():
    return (
        pkg_resources.EntryPoint.parse('a=data.plugins:FactoryA'),
    )


@mock.patch('pkg_resources.iter_entry_points')
def test_load_plugins(mock_pkg_resources):
    mock_pkg_resources.return_value = plugin_entry_points()

    plugin = plugins.load_plugins('a')

    assert plugin.versions
    assert plugin.definitions

    try:
        plugins.load_plugins('does_not_exist')

        assert False, 'Should have raised an KeyError'
    except KeyError:
        pass


@mock.patch('pkg_resources.iter_entry_points')
def test_install(mock_pkg_resources):
    mock_pkg_resources.return_value = plugin_entry_points()

    plugin = plugins.load_plugins('a')

    with disk.temp_directory() as temp_directory:
        plugin.install('1.2.3', temp_directory)

        p = plugin.load(temp_directory)

        assert p.version == '1.2.3'

        plugin.uninstall(temp_directory)


@mock.patch('pkg_resources.iter_entry_points')
def test_start(mock_pkg_resources):
    mock_pkg_resources.return_value = plugin_entry_points()

    plugin = plugins.load_plugins('a')

    with disk.temp_directory() as temp_directory:
        plugin.install('1.2.3', temp_directory)

        p = plugin.load(temp_directory)
        assert not p.running

        p.start()
        assert p.running
        p.stop()
        assert not p.running

        p.start()
        assert p.running
        p.kill()
        assert not p.running

        plugin.uninstall(temp_directory)



#!/usr/bin/env python
#
# Copyright 2014 the original author or authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

from io import open
import os
import subprocess
import sys

from setuptools import setup, Command

from cabalgata.silla import VERSION


class Doc(Command):
    description = 'generate or test documentation'
    user_options = [('test', 't', 'run doctests instead of generating documentation')]
    boolean_options = ['test']

    def initialize_options(self):
        self.test = False

    def finalize_options(self):
        pass

    def run(self):
        if self.test:
            path = 'docs/build/doctest'
            mode = 'doctest'
        else:
            path = 'docs/build/%s' % VERSION
            mode = 'html'

        try:
            os.makedirs(path)
        except OSError:
            pass

        status = subprocess.call(['sphinx-build', '-E', '-b', mode, '-d', 'docs/build/doctrees', 'docs/source', path])

        if status:
            raise RuntimeError('documentation step "%s" failed' % (mode,))

        sys.stdout.write('\nDocumentation step "%s" performed, results here:\n'
                         '   %s/\n' % (mode, path))


setup(
    name='cabalgata-silla-de-montar',
    version=VERSION,
    url='http://github.com/cabalgata/cabalgata-silla-de-montar/',
    license='Apache Software License (http://www.apache.org/licenses/LICENSE-2.0)',
    author='Alan D. Cabrera',
    author_email='adc@toolazydogs.com',
    description='A simple framework for installing and starting services.',
    # don't ever depend on refcounting to close files anywhere else
    long_description=open('README.rst', encoding='utf-8').read(),

    scripts=['bin/cabalgata'],

    packages=['cabalgata',
              'cabalgata/silla',
              'cabalgata/silla/util'],

    zip_safe=False,
    platforms='any',

    tests_require=['tox'],

    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    cmdclass={'doc': Doc},
)
